<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <link rel='shortcut icon' type='image/x-icon' href='../static/favicon.ico'/>
    <link rel = "stylesheet" type = "text/css" href = "{{url_for('static', filename='style.css')}}">
    {% if song and artist %}
        <title>Lyrics for {{song}}</title>
    {% else %}
        <title>Nothing Playing</title>
    {% endif %}
</head>
<body>
{% if song and artist %}
    <h2>{{song}} by {{artist}}</h2>
{% else %}
    <h2>Nothing Playing.</h2>
{% endif %}
<div id="lyrics">
{% for line in lyrics %}
{{line}}<br>
{% endfor %}
    </div>
<script type="text/javascript" src="{{url_for('static', filename='songChanged.js')}}">
</script>
</body>
</html>
